<?php

namespace Drupal\monarch_view_mode_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\monarch_view_mode_field\Plugin\Field\FieldType\ViewModeItem;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceEntityFormatter;

/**
 * Implementation of the 'entity reference entity view with override' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_entity_view_with_override",
 *   label = @Translation("Rendered entity with view mode override"),
 *   description = @Translation("Display the referenced entities rendered by entity_view()."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceEntityWithOverrideFormatter extends EntityReferenceEntityFormatter {

  /**
   * The entity_field.manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Returns the template details for this formatter.
   *
   * @return array
   *   The theme template details.
   */
  public static function theme() {
    return [
      'rendered_entity_link_wrapper' => [
        'variables' => [
          'url' => NULL,
          'title' => NULL,
          'content' => NULL,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->entityFieldManager = $container->get('entity_field.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'field_name' => '',
      'link_field_name' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['view_mode']['#title'] = $this->t('Default View mode');

    $elements['field_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Field to use for view mode override'),
      '#default_value' => $this->getSetting('field_name'),
      '#options' => [
        '' => $this->t('-- Any View Mode Field ---'),
      ],
    ];

    $elements['link_field_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Link to'),
      '#default_value' => $this->getSetting('link_field_name'),
      '#options' => [
        '' => $this->t('-- No Link --'),
        '-' => $this->t('-- Referenced Entity Url --'),
        '+' => $this->t('-- Field Parent Entity Url --'),
      ],
    ];

    foreach ($this->entityFieldManager->getFieldDefinitions($this->fieldDefinition->getTargetEntityTypeId(), $this->fieldDefinition->getTargetBundle()) as $field_name => $config) {
      if ($config->getType() === 'monarch_view_mode') {
        $elements['field_name']['#options'][$field_name] = $config->getLabel() . ' (' . $field_name . ')';
      }

      if ($config->getType() === 'link') {
        $elements['link_field_name']['#options'][$field_name] = $config->getLabel() . ' (' . $field_name . ')';
      }
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $view_mode = $this->getSetting('view_mode');
    $override_field_name = $this->getSetting('field_name');
    $link_field_name = $this->getSetting('link_field_name');
    $parent_entity = $items->getEntity();
    $link_item = NULL;

    if ($link_field_name && $link_field_name !== '-' && $link_field_name !== '+') {
      if ($parent_entity->hasField($link_field_name)) {
        foreach ($parent_entity->get($link_field_name) as $link_item) {
          break;
        }
      }
    }

    foreach ($items->getEntity()->getFields() as $field_name => $field_data) {
      if (!$override_field_name || $field_name === $override_field_name) {
        $field_item = $field_data[0] ?? NULL;

        if ($field_item instanceof ViewModeItem) {
          if ($field_item->value ?? NULL) {
            $possible_view_mode = explode('.', $field_item->value)[1] ?? NULL;

            if ($possible_view_mode) {
              $view_mode = $possible_view_mode;
              break;
            }
          }
        }
      }
    }

    $elements = [];

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      // Due to render caching and delayed calls, the viewElements() method
      // will be called later in the rendering process through a '#pre_render'
      // callback, so we need to generate a counter that takes into account
      // all the relevant information about this field and the referenced
      // entity that is being rendered.
      $recursive_render_id = $items->getFieldDefinition()->getTargetEntityTypeId()
        . $items->getFieldDefinition()->getTargetBundle()
        . $items->getName()
        // We include the referencing entity, so we can render default images
        // without hitting recursive protections.
        . $items->getEntity()->id()
        . $entity->getEntityTypeId()
        . $entity->id();

      if (isset(static::$recursiveRenderDepth[$recursive_render_id])) {
        static::$recursiveRenderDepth[$recursive_render_id]++;
      }
      else {
        static::$recursiveRenderDepth[$recursive_render_id] = 1;
      }

      // Protect ourselves from recursive rendering.
      if (static::$recursiveRenderDepth[$recursive_render_id] > static::RECURSIVE_RENDER_LIMIT) {
        $this->loggerFactory->get('entity')->error('Recursive rendering detected when rendering entity %entity_type: %entity_id, using the %field_name field on the %parent_entity_type:%parent_bundle %parent_entity_id entity. Aborting rendering.', [
          '%entity_type' => $entity->getEntityTypeId(),
          '%entity_id' => $entity->id(),
          '%field_name' => $items->getName(),
          '%parent_entity_type' => $items->getFieldDefinition()->getTargetEntityTypeId(),
          '%parent_bundle' => $items->getFieldDefinition()->getTargetBundle(),
          '%parent_entity_id' => $items->getEntity()->id(),
        ]);
        return $elements;
      }

      $elements[$delta] = [
        '#theme' => 'rendered_entity_link_wrapper',
      ];

      if ($link_item) {
        $elements[$delta]['#url'] = $link_item->getUrl()->toString();
        $elements[$delta]['#title'] = $link_item->title ?? NULL;
      }
      elseif ($link_field_name === '-') {
        try {
          $elements[$delta]['#url'] = $entity->toUrl()->toString();
        }
        catch (\Throwable $err) {
          // Just skip it.
        }
      }
      elseif ($link_field_name === '+') {
        try {
          $elements[$delta]['#url'] = $parent_entity->toUrl()->toString();
        }
        catch (\Throwable $err) {
          // Just skip it.
        }
      }

      $view_builder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
      $elements[$delta]['#content'] = $view_builder->view($entity, $view_mode, $entity->language()->getId());

      // Add a resource attribute to set the mapping property's value to the
      // entity's url. Since we don't know what the markup of the entity will
      // be, we shouldn't rely on it for structured data such as RDFa.
      if (!empty($items[$delta]->_attributes) && !$entity->isNew() && $entity->hasLinkTemplate('canonical')) {
        $items[$delta]->_attributes += ['resource' => $entity->toUrl()->toString()];
      }
    }

    return $elements;
  }

}
