<?php

namespace Drupal\monarch_view_mode_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'monarch_view_mode' field widget.
 *
 * @FieldWidget(
 *   id = "monarch_view_mode",
 *   label = @Translation("View Mode"),
 *   field_types = {"monarch_view_mode"},
 * )
 */
class ViewModeWidget extends WidgetBase {

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity_display.repository service.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->entityDisplayRepository = $container->get('entity_display.repository');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'available_view_modes' => [],
    ] + parent::defaultSettings();
  }

  /**
   * Get the list of content entities.
   *
   * @return array
   *   The list of view modes.
   */
  protected function getModeList() {
    $list = [];

    foreach ($this->entityTypeManager->getDefinitions() as $definition) {
      if ($definition instanceof ContentEntityTypeInterface) {
        foreach ($this->entityDisplayRepository->getViewModes($definition->id()) as $mode_id => $data) {
          $list[$definition->id()]['modes'][$definition->id() . '.' . $mode_id] = $data['label'];
        }

        if ($list[$definition->id()]['modes'] ?? NULL) {
          $list[$definition->id()]['label'] = $definition->getLabel() . ' (' . $definition->id() . ')';
        }
      }
    }

    return $list;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['available_view_modes'] = [
      '#type' => 'select',
      '#title' => $this->t('Available View Modes'),
      '#default_value' => $this->getSetting('available_view_modes'),
      '#multiple' => TRUE,
      '#size' => 10,
      '#options' => [
        'All' => [
          '' => $this->t('Default'),
        ],
      ],
    ];

    foreach ($this->getModeList() as $entity_type_id => $data) {
      foreach ($data['modes'] as $mode_id => $mode_label) {
        $element['available_view_modes']['#options'][$data['label']][$mode_id] = $mode_label;
      }
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $available_view_modes = $this->getSetting('available_view_modes');

    $element['value'] = $element + [
      '#type' => 'select',
      '#title' => $this->t('View Mode'),
      '#default_value' => $items[$delta]->value ?? NULL ?: '',
      '#options' => (empty($available_view_modes) || in_array('', $available_view_modes)) ? [
        'All' => [
          '' => $this->t('Default'),
        ],
      ] : [],
    ];

    foreach ($this->getModeList() as $data) {
      foreach ($data['modes'] as $mode_id => $mode_label) {
        if (empty($available_view_modes) || in_array($mode_id, $available_view_modes)) {
          $element['value']['#options'][$data['label']][$mode_id] = $mode_label;
        }
      }
    }

    if (count($element['value']['#options']) === 1) {
      $element['value']['#options'] = current($element['value']['#options']);
    }

    return $element;
  }

}
